#include "config.h"
#include <stdio.h>
#include "include/cpuid.h"

#ifdef buildvsnet
namespace vcnetspace
{
	extern int __cdecl dndmain(int argc, char** argv);
}
#endif

#ifdef builddebug
namespace debugspace
{
	extern int __cdecl dndmain(int argc, char** argv);
}
#endif

#ifdef buildintel
namespace intelspace
{
	extern int __cdecl dndmain(int argc, char** argv);
}
#endif

#ifdef buildvectorc
namespace vectorcspace
{
	extern int __cdecl dndmain(int argc, char** argv);
}
#endif

#ifdef buildgcc
	extern "C" void __stdcall _Z14if_gcc_dndmaini(int dump);
#endif


int Runtime(int Num, int argc, char** argv)
{
	int Start = Num;
	goto skiprestart;

restart:
	Num++;
	Num %= 5;

	if (Num == Start)
	{
		printf("No suitable Runtime found\n");
		getchar();
		return(1);
	}

skiprestart:

	if (Num == 0)
	{
#ifdef builddebug
		printf("Running Debug Code\n");
		return(debugspace::dndmain(argc, argv));
#else
		printf("Debug Runtime not included, Trying another\n");
		goto restart;
#endif
	}

	if (Num == 1)
	{
#ifdef buildintel
		printf("Running Intel Compiler Optimized Code\n");
		if (HasSSE2() == false)
		{
			printf("CPU doesnt support SSE2, switching Runtime\n");
			goto restart;
		}
		return(intelspace::dndmain(argc, argv));
#else
		printf("Intel Compiler Runtime not included, Trying another\n");
		goto restart;
#endif
	}

	if (Num == 2)
	{
#ifdef buildvectorc
		printf("Running VectorC Compiler Optimized Code\n");
		if (isAthlon() == FALSE)
		{
			printf("No Athlon CPU found, switching Runtime\n");
			goto restart;
		}
		return(vectorcspace::dndmain(argc, argv));
#else
		printf("VectorC Runtime not included, Trying another\n");
		goto restart;
#endif
	}

	if (Num == 3)
	{
#ifdef buildgcc
		printf("Running GCC Compiler Generic Optimized Code\n");
		_Z14if_gcc_dndmaini(0);
		return(0);
#else
		printf("GCC Runtime not included, Trying another\n");
		goto restart;
#endif
	}

	if (Num == 4)
	{
#ifdef buildvsnet
		printf("Running Visual C++ .NET Compiler Optimized Code\n");
		return(vcnetspace::dndmain(argc, argv));
#else
		printf("Visual C++ .NET Runtime not included, Trying another\n");
		goto restart;
#endif
	}

	printf("Unknown Runtime\nExiting...\n");
	getchar();
	return(1);
}

int main(int argc, char** argv)
{
	int StdRun;

	printf("DND Chargen V. 0.8d Coded by qon in 2k4\n\n");

	if (HasSSE2())
	{
		printf("SSE2 capable CPU found, using Intel Compiler Runtime\n");
		StdRun = 1;
	}
	else if (isAthlon())
	{
		printf("Athlon CPU with 3DNow found, using AMD optimized VectorC Runtime\n");
		StdRun = 2;
	}
	else
	{
		printf("Neither SSE2 nor Athlon CPU found, using Microsoft Compiler Generic Runtime\n");
		StdRun = 4;
	}

	return(Runtime(StdRun, argc, argv));
}
